package com.ruoyi.web.controller.moonshot;


import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.FileType;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.moonshot.Client;
import com.ruoyi.common.utils.moonshot.vo.FileUploadResult;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.tool.domain.TUnifyFile;
import com.ruoyi.tool.service.ITUnifyFileService;
import com.ruoyi.web.controller.common.CommonController;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping("/moonshot/file")
public class MoonshotFileController extends BaseController {

    private static final Logger log = LoggerFactory.getLogger(CommonController.class);

    @Autowired
    private ServerConfig serverConfig;

    @Autowired
    private ITUnifyFileService tUnifyFileService;

    @Value("${moonshot.api_key}")
    private String API_KEY;

    /**
     * 通用上传请求（单个）
     */
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file) throws Exception
    {
        AjaxResult ajax = AjaxResult.success();
        try
        {
            if (API_KEY == null) {
                System.out.println("Please set MOONSHOT_API_KEY env");
                ajax.put("API_KEY","接口API_KEY不存在");
                return ajax;
            }
            Client client = new Client(API_KEY);

            FileUploadResult resuat = client.uploadFile(file);

            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileName;
            TUnifyFile tUnifyFile = new TUnifyFile();
            tUnifyFile.setUploadPath(url);
            tUnifyFile.setUploadName(fileName);
            tUnifyFile.setNewUploadName(FileUtils.getName(fileName));
            tUnifyFile.setUploadFormat(file.getOriginalFilename());
            tUnifyFile.setUploadType(String.valueOf(FileType.RESUME.ordinal()));//简历附件
            tUnifyFile.setCreateBy(getNickName());

            //moonshot AI 附件上传返回附件信息
            if(null != resuat.getError()){
                ajax.put("message", resuat.getError().getMessage());
                tUnifyFile.setStatus_details(resuat.getError().getMessage());
            }else if(resuat.getStatus().equals("ok")) {
                tUnifyFile.copyFrom(resuat,true);
            }

            tUnifyFileService.insertTUnifyFile(tUnifyFile);

            ajax.put("FileUploadResult", resuat);
            return ajax;
        }
        catch (Exception e)
        {
            return AjaxResult.error(e.getMessage());
        }
    }


}
